// log-handler.sma
//
// This library contains a log handler
//
// Copyright (c) 2018 Superbase Software Limited, all rights reserved.
// Author: Michael Ott, Nikolaus-Maxim Zolnhofer
//
// Included sml's:
// ===============
// logmanager.sml
//
// TODO:
//////////////////////////////////////////////////////////////////////////////////

constant LOG_HANDLER_DEBUG 0
constant LOG_HANDLER_TRACE 1
constant LOG_HANDLER_INFO  2
constant LOG_HANDLER_WARN  3
constant LOG_HANDLER_ERROR 4

// Class LogHandler
type LogHandler export
  embed
  string file_name;
  integer level;

  reference
  LogManager log_manager readonly

  function debug_message readonly
  function error_message readonly
  function warn_message readonly
  function info_message readonly
  function trace_message readonly

  // TODO privat function
  function message readonly
end type

// Constructor
// me: Pointer to LogHandler class
// file_name: Log file name 
// level: Log level (default: LOG_HANDLER_DEBUG)
function LogHandler.new(LogHandler me, string file_name, integer level = LOG_HANDLER_DEBUG)
  LogManager _log_manager; 

  !setproperty(me, me.type!file_name, file_name); 
  !setproperty(me, me.type!level, level); 

  _log_manager =@ LogManager.new(); 
  !setproperty(me, me.type!log_manager, _log_manager); 

end function me

// Write a debug message to log file
// me: Pointer to LogHandler class
// str: String to write
function LogHandler.debug_message(LogHandler me, string str)
end function me.message(LOG_HANDLER_DEBUG, str)

// Write a error message to log file
// me: Pointer to LogHandler class
// str: String to write
function LogHandler.error_message(LogHandler me, string str)
end function me.message(LOG_HANDLER_ERROR, str)

// Write a warn message to log file
// me: Pointer to LogHandler class
// str: String to write
function LogHandler.warn_message(LogHandler me, string str)
end function me.message(LOG_HANDLER_WARN, str)

// Write a info message to log file
// me: Pointer to LogHandler class
// str: String to write
function LogHandler.info_message(LogHandler me, string str)
end function me.message(LOG_HANDLER_INFO, str)

// Write a trace message to log file
// me: Pointer to LogHandler class
// str: String to write
function LogHandler.trace_message(LogHandler me, string str)
end function me.message(LOG_HANDLER_TRACE, str)

// Write a message to log file
// me: Pointer to LogHandler class
// level: Log level to use for this message
// str: String to write
function LogHandler.message(LogHandler me, integer level, string str)
  string result; 

  if level == LOG_HANDLER_DEBUG
    result = "Debug";
  else if level == LOG_HANDLER_TRACE
    result = "Trace"; 
  else if level == LOG_HANDLER_INFO
    result = "Info"; 
  else if level == LOG_HANDLER_WARN
    result = "Warn"; 
  else if level == LOG_HANDLER_ERROR
    result = "Error"; 
  end if

  result = result + ": " + str; 
  if level >= me.level
    me.log_manager.writeLogEntry(me.file_name, result); 
    // TODO remove wait
    //!wait(1000); 
  end if

end function result